state_machine = {color = nil, font = nil}
local states = {} -- The State Stack

-- State Template
function newState()
  local state_template = {}
  function state_template.load(vars)                  end
  function state_template.update(dt)                  end
  function state_template.draw()                      end
  function state_template.keypressed(key)             end
  function state_template.keyreleased(key)            end
  function state_template.textinput(t)                end
  function state_template.mousemoved(x, y, dx, dy)    end
  function state_template.mousepressed(x, y, button)  end
  function state_template.mousereleased(x, y, button) end
  function state_template.wheelmoved(x, y)            end
   
  return state_template
end

-- STACK-TYPE STATE MACHINE
function state_machine.load(default_color, default_font)
  state_machine.color = default_color or {1, 1, 1}
  state_machine.font = default_font or love.graphics.newFont()
  states = {}
end

function state_machine.push(state, vars)
  table.insert(states, state)
  states[#states].load(vars)
end
function state_machine.pop()
  table.remove(states, #states)
end
function state_machine.change(state, vars)
  states = {}
  state_machine.push(state, vars)
end

-- Update only the top-most State in the Stack (Useful for Pause States)
function state_machine.update(dt)
  if #states > 0 then
    states[#states].update(dt)
  end
end
-- Draw everything in the stack (Also Useful for Pause States)
function state_machine.draw()
  if #states > 0 then
    for i = 1, #states do
      love.graphics.setColor(state_machine.color)
      love.graphics.setFont(state_machine.font)
      states[i].draw()
    end
  end
end

-- Keyboard Events
function state_machine.keypressed(key)
  if #states > 0 then
    states[#states].keypressed(key)
  end
end
function state_machine.keyreleased(key)
  if #states > 0 then
    states[#states].keyreleased(key)
  end
end
function state_machine.textinput(t)
  if #states > 0 then
    states[#states].textinput(t)
  end
end

-- Mouse Events
function state_machine.mousemoved(x, y, dx, dy)
  if #states > 0 then
    states[#states].mousemoved(x, y, dx, dy)
  end
end
function state_machine.mousepressed(x, y, button)
  if #states > 0 then
    states[#states].mousepressed(x, y, button)
  end
end
function state_machine.mousereleased(x, y, button)
  if #states > 0 then
    states[#states].mousereleased(x, y, button)
  end
end
function state_machine.wheelmoved(x, y)
  if #states > 0 then
    states[#states].wheelmoved(x, y)
  end
end


