-- Preferrably uses Monospace Fonts
local blinkrate = 0.5

function newTextInput(font, x, y, maxlength, vars)
  local backspacetimer = 0.7
  local blinktimer = 0
  local blinkchar = '_'
  local minwrap = (font:getWidth('W') * maxlength) / 2 -- I think "W" is the widest character, but it's dynamic
  local wrap = minwrap

  local padding = vars and vars.padding     or 2
  local txcolor = vars and vars.textcolor   or {1,1,1}
  local bxcolor = vars and vars.boxcolor    or {0,0,0,0.5}
  local bdcolor = vars and vars.bordercolor or {1,1,1}
  local rx      = vars and vars.roundx      or 0
  local ry      = vars and vars.roundy      or 0

  local textbox = {
    x = x, y = y,
    text = vars and vars.text or '',
    active = true
  }
  
  function textbox:update(dt)
    if self.active then
      if self.text:len() < maxlength then
        blinktimer = blinktimer + dt
        if blinktimer >= blinkrate then
          blinktimer = 0
        
          -- Inline Condition: var = <cond> and <var_true> or <var_false>
          blinkchar = blinkchar == '_' and ' ' or '_'
        end
      else
        -- Don't blink when maxlength is reached
        blinkchar = ''
      end
      
      if love.keyboard.isDown('backspace') then
        backspacetimer = math.max(backspacetimer - dt, 0)
        if backspacetimer == 0 and self.text:len() > 0 then
          self.text = self.text:sub(0, self.text:len() - 1)
          local textwrap = font:getWidth(self.text .. '_')
          wrap = math.max(minwrap, textwrap)

          backspacetimer = 0.05
        end
      else
        backspacetimer = 0.7
      end
    end
  end
  
  function textbox:draw()
    if self.active then
      local h = font:getHeight('W') + (padding * 2)
      local w = wrap + (padding * 2)

      -- Filling the box
      love.graphics.setColor(bxcolor)
      love.graphics.rectangle('fill', x - padding, y - padding, w, h, rx, ry)    

      -- Same thing for the border, except 'line'
      love.graphics.setColor(bdcolor)
      love.graphics.rectangle('line', x - padding, y - padding, w, h, rx, ry)
      
      love.graphics.setFont(font)
      love.graphics.print({txcolor, self.text .. blinkchar}, x, y)
    end
  end
  
  function textbox:textinput(t)                           -- I assume that (\ ' ") will produce bugs
    if self.active and self.text:len() < maxlength and not (t == '\\' or t == "'" or t == '"') then
    
      -- Don't accept 'spacebar' if the text is empty, or if you previously entered 'space'
      if not (t == ' ' and (self.text == '' or self.text:sub(-1) == ' ')) then
        self.text = self.text .. t
      end
      
      -- Re-adjust display to textlength
      local textwrap = self.text:len() == maxlength and font:getWidth(self.text) or font:getWidth(self.text .. '_')
      wrap = math.max(minwrap, textwrap)
    end
  end
  
  function textbox:keypressed(key)
    if self.active and key == 'backspace' then
      self.text = self.text:sub(0, self.text:len() - 1)
      local textwrap = font:getWidth(self.text .. '_')
      wrap = math.max(minwrap, textwrap)
    end
  end
  
  return textbox
end
