require 'lib/screen'
require 'lib/statemachine'
require 'lib/menu'
require 'lib/textinput'
require 'lib/entities'

require 'src/objects/notations'
require 'src/objects/player'
require 'src/objects/board'
require 'src/objects/guiboard'
require 'src/objects/ai'

require 'src/states/title'
require 'src/states/selectboard'
require 'src/states/selectplayers'
require 'src/states/play'
require 'src/states/load'
require 'src/states/tutorial'
require 'src/states/save'
require 'src/states/pause'

function love.load()
  -- Don't blur images when zoomed in/out
  love.graphics.setDefaultFilter('nearest', 'nearest')
  
  -- Force Resolution No Matter the Window Size
  screen.set(320,224) -- Sega Genesis Resolution
  
  -- Globals
  menu = newMenu(love.graphics.newImageFont('font/font2.png', ' ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890._:'), 2,
    {
      textcolor       = { 240/255, 240/255,       0},
      buttoncolor     = {  90/255, 110/255, 150/255, 0.3},
      selectedcolor   = {  70/255, 180/255, 200/255, 0.8},
      registeredcolor = {  50/255,       1,       1, 0.8},
      bordercolor     = {  71/255, 138/255, 175/255},
      roundx = 1,
      roundy = 1,
    }
  )
  
  -- Universal board
  board = newGuiBoard(1 ,'G', 1)

  -- My main.lua is heavily tied to state_machine
  state_machine.load(nil, love.graphics.newImageFont('font/font3.png',' ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890._:'))
  state_machine.push(state_title)
end

function love.update(dt)
  board:update(dt)
  state_machine.update(dt)
end
function love.draw()
  -- Forced Resolution Draw
  screen.draw()

  -- Put universal draws here
  love.graphics.setColor(1,1,1)
  board:draw()
  
  -- State Machine
  state_machine.draw()
end

-- Keyboard Inputs
function love.keypressed(key)
  state_machine.keypressed(key)
end
function love.keyreleased(key)
  state_machine.keyreleased(key)
end
function love.textinput(t)
  state_machine.textinput(t)
end

-- Mouse Functions
function love.mousemoved(x, y, dx, dy)
  state_machine.mousemoved(screen.convertX(x), screen.convertY(y), dx, dy)
end
function love.mousepressed(x, y, button)
  state_machine.mousepressed(screen.convertX(x), screen.convertY(y), button)
end
function love.mousereleased(x, y, button)
  state_machine.mousereleased(screen.convertX(x), screen.convertY(y), button)
end
function love.wheelmoved(x, y)
  state_machine.wheelmoved(x, y)
end

-- Window Functions
function love.resize(w, h)
  screen.resize(w, h)
end
