ai = {}

local board = nil
local walkers = {}
local sliders = {}
local path = {}
local target = 0

local function checkSliders(index, w_indexes, s_indexes)
  table.insert(w_indexes, index)

  local test = index - 1 -- Adjacent Index Checker
  local checked = false  -- Checkes if test is already in w_indexes
  local flag = false     -- Checks if test is a sliding tile

  -- Left
  if board:getFile(index) > 1 then
    for _, v in pairs(w_indexes) do
      if v == test then checked = true end
    end
    
    if checked == false then
      if board.squares[test] == notations.slider then
        checkSliders(test, w_indexes, s_indexes)
      elseif board.squares[test] == notations.rink or board.squares[test] == notations.goal then
        flag = true -- Flag now serves as an indicator that this is a sliding tile
      end
    end
  end

  -- Up  
  if board:getRank(index) > 1 then
    test = index - board.files
    checked = false
    -- No internal difference with "Left"
    for _, v in pairs(w_indexes) do
      if v == test then checked = true end
    end
    
    if checked == false then
      if board.squares[test] == notations.slider then
        checkSliders(test, w_indexes, s_indexes)
      elseif board.squares[test] == notations.rink or board.squares[test] == notations.goal then
        flag = true
      end
    end
  end
  
  -- Right
  if board:getFile(index) < board.files then
    test = index + 1
    checked = false
    for _, v in pairs(w_indexes) do
      if v == test then checked = true end
    end
    
    if checked == false then
      if board.squares[test] == notations.slider then
        checkSliders(test, w_indexes, s_indexes)
      elseif board.squares[test] == notations.rink or board.squares[test] == notations.goal then
        flag = true
      end
    end
  end
  
  -- Down
  if board:getRank(index) < board.ranks then
    test = index + board.files
    checked = false
    for _, v in pairs(w_indexes) do
      if v == test then checked = true end
    end
    
    if checked == false then
      if board.squares[test] == notations.slider then
        checkSliders(test, w_indexes, s_indexes)
      elseif board.squares[test] == notations.rink or board.squares[test] == notations.goal then
        flag = true
      end
    end
  end
  
  if flag == true then
    table.insert(s_indexes, index)
  end

  return w_indexes, s_indexes
end

local function findPath(index, target, path, checked_indexes)
  -- Insert index to path
  table.insert(path, index)

  -- Remove index from checked indexes
  for i = #checked_indexes, 1, -1 do
    if checked_indexes[i] == index then
      table.remove(checked_indexes, i)
    end
  end

  local target_found = index == target
  local test = index - 1
  local transfer = false
  
  -- Left
  if target_found == false then
    for _,v in pairs(checked_indexes) do
      if v == test then transfer = true end
    end
    
    if transfer then
      _, target_found = findPath(test, target, path, checked_indexes)
    end
  end
  -- Up
  if not target_found then
    test = index - board.files
    transfer = false
    for _,v in pairs(checked_indexes) do
      if v == test then transfer = true end
    end
    
    if transfer then
      _, target_found = findPath(test, target, path, checked_indexes)
    end
  end
  -- Right
  if not target_found then
    test = index + 1
    transfer = false
    for _,v in pairs(checked_indexes) do
      if v == test then transfer = true end
    end
    
    if transfer then
      _, target_found = findPath(test, target, path, checked_indexes)
    end
  end
  -- Down
  if not target_found then
    test = index + board.files
    transfer = false
    for _,v in pairs(checked_indexes) do
      if v == test then transfer = true end
    end
    
    if transfer then
      _, target_found = findPath(test, target, path, checked_indexes)
    end
  end
  
  if not target_found then table.remove(path, #path) end
  
  return path, target_found
end

function ai:update(b, index, dt)
  local from, to, slide = 0, 0, false
  
  -- Phase 1: Get Board positions
  if board == nil then
    board = b
    walkers = {}
    sliders = {}
    path = {}
    target = 0

  -- Phase 2: Check for adjascent sliders
  elseif #walkers == 0 then
    walkers, sliders = checkSliders(index, {}, {})

  -- Phase 3: Choose a target index
  elseif target == 0 then
    if #sliders > 0 then
      target = sliders[love.math.random(#sliders)]
    else
      target = walkers[love.math.random(#walkers)]
    end
  
  -- Phase 4: Get the path to target
  elseif #path == 0 then
    path = findPath(index, target, {}, walkers)

  -- Phase 5: Choosing move until you get to target index (last item)
  elseif #path > 1 then
    from = path[1]
    to = path[2]
    table.remove(path, 1)

  -- Phase 6: Sliding move (Even in passes)
  else
    slide = true
    from = path[1]
    if #sliders > 0 then
      -- Find a sliding location
      local moves = board:generateMoves(index)
      local destinations = {}
      local v = moves['up']
      if v.index > 0 and v.slide then table.insert(destinations, v.index) end
      v = moves['down']
      if v.index > 0 and v.slide then table.insert(destinations, v.index) end
      v = moves['left']
      if v.index > 0 and v.slide then table.insert(destinations, v.index) end
      v = moves['right']
      if v.index > 0 and v.slide then table.insert(destinations, v.index) end
      
      to = destinations[love.math.random(#destinations)]
    else
      to = path[1]
    end
    board = nil
  end
  return from, to, slide
end

function ai:draw()
  for i, v in pairs(path) do
    love.graphics.print(v, 0, i * 10)
  end
end

--[[

Ang Basic Cloud Types

Mag-umpisa tayo sa tatlo:
1. Ang madalas nating iguhit na Cumulus
2. Ang mahaba at walang definite shape na Stratus
3. At ang tila winalis na Cirrus

Ngayon, ang combination nila:
Cirrocumulus, Cirrostratus, at Stratocumulus

Bye-bye muna tayo kay cirrus at punta tayo sa Alto-type Clouds
Alto-cumulus, at Alto-stratus

Next, ang Nimbus-types or Rainclouds:
Cumulonimbus, at Nimbostratus

Kapag tumingala tayo sa himpapawid, pwede rin nating makilala ang mga ito base sa taas ng lipad:
Ang Cirrus-types ang pinaka-mataas
Nasa ilalim nila ang Alto-types,
And the rest, malapit lang sila sa atin.

At dahi hindi na ako estudyante, hindi ko na kailangang i-memorize ito. It's not my problem anymore, it's YOUR problem anymore.
Pero share ko lang.
--]]