state_play = newState()

local initial_rep, speed, moving_slider, moving_player, ai_players, direction

local walkspeed, slidespeed = 100, 175
local tile_width, tile_height = 32, 30

local function initializeMoves()
  board.can_slide = board:checkSliders(board.players[board.turn_counter].index, {})
  board.moves = board:generateMoves(board.players[board.turn_counter].index)
  board:loadMoves()
  state = 'think'
end

local function prepareMovement(is_sliding)
  board.draw_moves = false
  board.can_slide = true
  for _, v in pairs(board.sliders) do v:setAnimation('idle') end
  board.players[moving_player]:setAnimation(is_sliding and 'idle' or 'walking', not is_sliding)
  speed = is_sliding and slidespeed or walkspeed
  
  if board.players[moving_player].x < board.players[moving_player].target_x then
    board.players[moving_player].y = board.players[moving_player].target_y
    direction = 'right'
  elseif board.players[moving_player].x > board.players[moving_player].target_x then
    board.players[moving_player].y = board.players[moving_player].target_y
    direction = 'left'
  elseif board.players[moving_player].y < board.players[moving_player].target_y then
    board.players[moving_player].x = board.players[moving_player].target_x
    direction = 'down'
  else
    board.players[moving_player].x = board.players[moving_player].target_x
    direction = 'up'
  end
  
  state = 'moving'
end

local function tryToMove(direction)
  if board.moves[direction].index > 0 then
    moving_slider, moving_player = board:movePiece(board.players[board.turn_counter].index, board.moves[direction].index, board.moves[direction].slide)
    prepareMovement(moving_slider > 0)
    board.turn_counter = moving_slider > 0 and (board.turn_counter % #board.players) + 1 or board.turn_counter
  end
end

local function passMove()
  moving_slider, moving_player = board:movePiece(board.players[board.turn_counter].index, board.players[board.turn_counter].index, true)
  prepareMovement(true)
  board.turn_counter = (board.turn_counter % #board.players) + 1
end

local function restartGame()
  board = newGuiBoard(board.files, initial_rep, board.goal_index)
  for _, v in pairs(ai_players) do
    board.players[v].ai = true
  end
  initializeMoves()
end

function state_play.load()
  initial_rep = board:getRepresentation()
  ai_players = {}
  for i, v in pairs(board.players) do
    if v.ai then table.insert(ai_players, i) end
  end
  menu.active = false
  
  initializeMoves()
end

function state_play.update(dt)
  if state == 'think' then
    if board.players[board.turn_counter].ai and board.history_position == #board.move_history then
      local from, to, slide = ai:update(board, board.players[board.turn_counter].index, dt)
      if from > 0 then
        moving_slider, moving_player = board:movePiece(from, to, slide)
        prepareMovement(moving_slider > 0)
        board.turn_counter = slide and (board.turn_counter % #board.players) + 1 or board.turn_counter
      end
    else
      if love.keyboard.isDown('up') and board.moves['up'].slide == false then
        tryToMove('up')
      end
      if love.keyboard.isDown('left') and board.moves['left'].slide == false then
        tryToMove('left')
      end
      if love.keyboard.isDown('right') and board.moves['right'].slide == false then
        tryToMove('right')
      end
      if love.keyboard.isDown('down') and board.moves['down'].slide == false then
        tryToMove('down')
      end
    end
  elseif state == 'moving' then
    local player = board.players[moving_player]
    local flag = false
    if board.players[moving_player].y > player.target_y then
      board.players[moving_player].y = math.max(player.y - speed * dt, player.target_y)
    elseif direction == 'down' and player.y < player.target_y then
      board.players[moving_player].y = math.min(player.y + speed * dt, player.target_y)
    elseif direction == 'left' and player.x > player.target_x then
      board.players[moving_player].x = math.max(player.x - speed * dt, player.target_x)
    elseif direction == 'right' and player.x < player.target_x then
      board.players[moving_player].x = math.min(player.x + speed * dt, player.target_x)
    else
      flag = true
      if moving_slider > 0 then board:bump() end
    end

    if moving_slider > 0 then
      board.sliders[moving_slider].x = board.players[moving_player].x
      board.sliders[moving_slider].y = board.players[moving_player].y
    end

    if flag then

      flag = board:evaluateWinner()

      if flag == 0 then -- If no winner has been detected yet
        board.players[moving_player]:setAnimation('idle', false)
        initializeMoves()
      else
        -- END GAME
        for i, v in ipairs(board.players) do
          v:setAnimation(i == flag and 'victory' or 'defeat')
        end
        board:win()

        menu.buttons = {}
        menu:addButton('RESTART', screen.width / 2, screen.height / 2 - 22, function() restartGame() end)
        menu:addButton('SAVE',    screen.width / 2, screen.height / 2,      function() state_machine.push(state_save) end)
        menu:addButton('QUIT'   , screen.width / 2, screen.height / 2 + 22, function() state_machine.change(state_title) end)
        menu.active = true

        state = 'end'
      end
    end
  elseif state == 'end' then
    -- To Be Coded
  end
  board:update(dt)
end

function state_play.draw()
  if state == 'end' then menu:draw() end
end

function state_play.keypressed(key)
  if key == 'escape' and state ~= 'end' then
    state_machine.push(state_pause)
  elseif state == 'think' then
  
    -- Space = Pass (Only if you can't slide)
    if key == 'space' and board.can_slide == false and not board.players[board.turn_counter].ai then
      passMove()
    -- Moving to a direction
    elseif (key == 'up' or key == 'down' or key == 'left' or key == 'right')
    and board.moves[key].slide and not board.players[board.turn_counter].ai then
      tryToMove(key)
    -- Redo Move
    elseif key == 'x' then
      moving_slider, moving_player = board:redoMove()
      if moving_player > 0 then
        prepareMovement(moving_slider > 0)
        board.turn_counter = moving_slider > 0 and (board.turn_counter % #board.players) + 1 or board.turn_counter
      end
    -- Undo Move
    elseif key == 'z' then
      moving_slider, moving_player = board:undoMove()
      if moving_player > 0 then
        board.players[board.turn_counter]:setAnimation('idle', false)
        prepareMovement(moving_slider > 0)
        if moving_slider > 0 then
          board.turn_counter = board.turn_counter == 1 and #board.players or board.turn_counter - 1 
        end
      end
    end
  elseif state == 'end' then
    if key == 'return' or key == 'space' or key == 'kpenter' then
      menu:press()
    elseif key == 'up' or key == 'left' then
      menu:cycleUp()
    elseif key == 'down' or key == 'right' then
      menu:cycleDown()
    end
  end
end

function state_play.keyreleased(key)
  if state == 'end' then
    menu:release()
  end
end