state_save = newState()

local input, message, timer, suggestion

local function tryToSave()
  -- Remove an extra space in the end
  if input.text:sub(-1) == ' ' then
    input.text = input.text:sub(0, input.text:len() - 1)
  end
  
  -- If it's an empty text, use suggested name
  input.text = input.text == '' and suggestion or input.text
  
  -- If the file already exists...
  if love.filesystem.getInfo(input.text .. '.gam') then
    message = 'Name already exists!'
    
  -- Otherwise, save the game
  else
    local data = board.files  .. '\n' ..
      board:getRepresentation() .. '\n' ..
      board.goal_index  .. '\n' ..
      board.turn_counter .. '\n' ..
      board.history_position .. '\n'
    
    local players = ''
    for i = 1, #board.players do
      players = players .. (board.players[i].ai and '1' or '0') .. ' '
    end

    data = data .. players .. '\n'
    
    for i = 1, #board.move_history do
      local v = board.move_history[i]
      data = data .. v.from .. ' ' .. v.to .. ' ' .. (v.slide and 'true' or 'false') .. '\n'
    end

    love.filesystem.write(input.text .. '.gam', data)

    -- Leave State
    state_machine.pop()
  end
end

function state_save.load()
  -- Make sure that we give it a newly-saved Game name
  local i = 1
  while love.filesystem.getInfo('Game ' .. i .. '.gam') do
    i = i + 1
  end
  
  input = newTextInput(state_machine.font,
    screen.width / 2 - (state_machine.font:getWidth('w') * 4),
    40,
    15,
    {text = 'Game ' .. i}
  )
  suggestion = input.text
  message = ''
  timer = 3
end

function state_save.update(dt)
  input:update(dt)
  if message ~= '' then
    timer = timer - dt
    if timer <= 0 then
      timer = 3
      message = ''
    end
  end
end

function state_save.draw()
  love.graphics.printf('SAVE GAME', 0, 20, screen.width, 'center')
  input:draw()
  love.graphics.setColor(200/255, 0, 0)
  love.graphics.printf(message, 0, 60, screen.width, 'center')
  
end

function state_save.keypressed(key)
  if key == 'escape' then
    state_machine.pop()
  elseif key == 'return' then
    tryToSave()
  else
    input:keypressed(key)
  end
end

function state_save.textinput(t)
  input:textinput(t)
end