local pseudobuttons = require 'src/objects/pseudobuttons'

state_selectboard = newState()

local boards, selected_index, left, right

function state_selectboard.load(index)
  -- Get all items in "files" folder
  local folder_items = love.filesystem.getDirectoryItems('files')

  -- List Down only files that end in ".brd"
  local list = {}
  for _, file in pairs(folder_items) do
    if file:sub(-4) == '.brd' then
      table.insert(list, file)
    end
  end
  
  -- Fill our "boards" table with the necessary stuff from the "list" of files
  boards = {}
  for _, brdfile in pairs(list) do
    -- 1. For each ".brd file", insert all it's "lines" in "data"
    local data = {}
    for line in love.filesystem.lines('files/' .. brdfile) do
      table.insert(data, line)
    end
    
    -- 2. Just in case the file has lines more than 3
    local representation = ''
    for i = 2, #data - 1 do
      representation = representation .. data[i]
    end
    
    -- 2. Inserting that data into "boards" table
    table.insert(boards, {
      files = tonumber(data[1]),
      representation = representation,
      goal_index = tonumber(data[#data])
    })
  end

  selected_index = index or 1
  local s = boards[selected_index]
  board = newGuiBoard(s.files, s.representation, s.goal_index)
  
  -- Pseudo-button System
  left, right = newPseudoButtons()
  
  -- Global Menu
  menu.buttons = {}
  menu:addButton('SELECT', screen.width / 2, screen.height / 2 + 22, function() state_machine.change(state_selectplayers, selected_index)  end)
  menu:addButton('BACK',   screen.width / 2, screen.height / 2 + 44, function() state_machine.change(state_title) end)
  menu.selected_index = 1
end

function state_selectboard.update(dt)
  left:setAnimation(love.keyboard.isDown('left') and 'leftdown' or 'leftup')
  right:setAnimation(love.keyboard.isDown('right') and 'rightdown' or 'rightup')
end

function state_selectboard.draw()
  love.graphics.printf('Select Board', 0, 20, screen.width, 'center')
  left:draw()
  right:draw()
  menu:draw()
end

function state_selectboard.keypressed(key)
  if key == 'escape' then
    state_machine.change(state_title)
  elseif key == 'return' or key == 'space' or key == 'kpenter' then
    menu:press()
  elseif key == 'up' then
    menu:cycleUp()
  elseif key == 'down' then
    menu:cycleDown()
  else
    if key == 'left' then
      selected_index = selected_index == 1 and #boards or selected_index - 1
    elseif key == 'right' then
      selected_index = selected_index == #boards and 1 or selected_index + 1
    end
    local s = boards[selected_index]
    board = newGuiBoard(s.files, s.representation, s.goal_index)
  end
end

function state_selectboard.keyreleased(key)
  menu:release()
end